@@ -40,7 +40,7 @@
 #include "menubar.h"
 #include "spell.h"
 
-#define CONTROL_IID "OAFIID:GNOME_Spell_Control:0.2"
+#define CONTROL_IID "OAFIID:GNOME_Spell_Control:0.3"
 
 void
 spell_suggestion_request (GtkHTML *html, gpointer data)
@@ -53,7 +53,7 @@ spell_suggestion_request (GtkHTML *html, gpointer data)
 GNOME_Spell_Dictionary
 spell_new_dictionary (void)
 {
-#define DICTIONARY_IID "OAFIID:GNOME_Spell_Dictionary:0.2"
+#define DICTIONARY_IID "OAFIID:GNOME_Spell_Dictionary:0.3"
 
 	GNOME_Spell_Dictionary dictionary = bonobo_get_object (DICTIONARY_IID, "GNOME/Spell/Dictionary", NULL);
 
@@ -101,8 +101,19 @@ spell_add_to_session (GtkHTML *html, const gchar *word, gpointer data)
 	CORBA_exception_free (&ev);
 }
 
+static void
+spell_add_to_personal_no_language ()
+{
+	GtkWidget *info;
+
+	info = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
+				       GTK_BUTTONS_OK, _("Unable to add word to dictionary,\nlanguage settings are broken.\n"));
+	gtk_dialog_run (GTK_DIALOG (info));
+	gtk_widget_destroy (info);
+}
+
 void
-spell_add_to_personal (GtkHTML *html, const gchar *word, gpointer data)
+spell_add_to_personal (GtkHTML *html, const gchar *word, const gchar *language, gpointer data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
 	CORBA_Environment   ev;
@@ -112,9 +123,12 @@ spell_add_to_personal (GtkHTML *html, const gchar *word, gpointer data)
 	if (!cd->dict)
 		return;
 
-	CORBA_exception_init (&ev);
-	GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, word, &ev);
-	CORBA_exception_free (&ev);
+	if (language) {
+		CORBA_exception_init (&ev);
+		GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, word, language, &ev);
+		CORBA_exception_free (&ev);
+	} else
+		spell_add_to_personal_no_language ();
 }
 
 void
@@ -133,7 +147,10 @@ spell_set_language (GtkHTML *html, const gchar *language, gpointer data)
 	GNOME_Spell_Dictionary_setLanguage (cd->dict, language, &ev);
 	CORBA_exception_free (&ev);
 
-	menubar_set_languages (cd, language);
+	g_free (cd->language);
+	cd->language = g_strdup (language);
+
+	menubar_set_languages (cd);
 }
 
 void
@@ -154,10 +171,12 @@ set_word (GtkHTMLControlData *cd)
 }
 
 static gboolean
-next_word (GtkHTMLControlData *cd)
+next_word (GtkHTMLControlData *cd, gboolean forward)
 {
 	gboolean rv = TRUE;
-	while (html_engine_forward_word (cd->html->engine)
+	if (!forward)
+		html_engine_backward_word (cd->html->engine);
+	while ((forward ? html_engine_forward_word (cd->html->engine) : html_engine_backward_word (cd->html->engine))
 	       && (rv = html_engine_spell_word_is_valid (cd->html->engine)))
 		;
 
@@ -165,7 +184,7 @@ next_word (GtkHTMLControlData *cd)
 }
 
 static void
-check_next_word (GtkHTMLControlData *cd, gboolean update)
+check_next_word (GtkHTMLControlData *cd, gboolean update, gboolean forward)
 {
 	HTMLEngine *e = cd->html->engine;
 
@@ -173,7 +192,7 @@ check_next_word (GtkHTMLControlData *cd, gboolean update)
 	if (update)
 		html_engine_spell_check (e);
 
-	if (!cd->spell_check_next || next_word (cd)) {
+	if (!cd->spell_check_next || next_word (cd, forward)) {
 		gtk_dialog_response (GTK_DIALOG (cd->spell_dialog), GTK_RESPONSE_CLOSE);
 	} else {
 		set_word (cd);
@@ -186,27 +205,39 @@ replace_cb (BonoboListener *listener, const char *event_name, const CORBA_any *a
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
 
 	html_engine_replace_spell_word_with (cd->html->engine, BONOBO_ARG_GET_STRING (arg));
-	check_next_word (cd, FALSE);
+	check_next_word (cd, FALSE, TRUE);
 }
 
-static void 
+static void
 skip_cb (BonoboListener *listener, const char *event_name, const CORBA_any *arg, CORBA_Environment *ev, gpointer user_data)
 {
-	check_next_word ((GtkHTMLControlData *) user_data, FALSE);
+	check_next_word ((GtkHTMLControlData *) user_data, FALSE, TRUE);
+}
+
+static void
+back_cb (BonoboListener *listener, const char *event_name, const CORBA_any *arg, CORBA_Environment *ev, gpointer user_data)
+{
+	check_next_word ((GtkHTMLControlData *) user_data, FALSE, FALSE);
 }
 
 static void 
 add_cb (BonoboListener *listener, const char *event_name, const CORBA_any *arg, CORBA_Environment *ev, gpointer user_data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
-	gchar *word;
+	gchar *word, *language;
 
 	word = html_engine_get_spell_word (cd->html->engine);
 	g_return_if_fail (word);
 
-	GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, word, ev);
+	language = BONOBO_ARG_GET_STRING (arg);
+
+	if (language) {
+		GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, word, language, ev);
+		check_next_word ((GtkHTMLControlData *) user_data, TRUE, TRUE);
+	} else
+		spell_add_to_personal_no_language ();
+
 	g_free (word);
-	check_next_word ((GtkHTMLControlData *) user_data, TRUE);
 }
 
 static void 
@@ -220,7 +251,7 @@ ignore_cb (BonoboListener *listener, const char *event_name, const CORBA_any *ar
 
 	GNOME_Spell_Dictionary_addWordToSession (cd->dict, word, ev);
 	g_free (word);
-	check_next_word ((GtkHTMLControlData *) user_data, TRUE);
+	check_next_word ((GtkHTMLControlData *) user_data, TRUE, TRUE);
 }
 
 gboolean
@@ -244,23 +275,31 @@ spell_check_dialog (GtkHTMLControlData *cd, gboolean whole_document)
 	GtkWidget *control;
 	GtkWidget *dialog;
 	guint position;
+	gboolean inline_spelling = gtk_html_get_inline_spelling (cd->html);
 
 	position = cd->html->engine->cursor->position;
 	cd->spell_check_next = whole_document;
 	if (whole_document) {
 		html_engine_disable_selection (cd->html->engine);
 		html_engine_beginning_of_document (cd->html->engine);
+		if (!inline_spelling)
+			gtk_html_set_inline_spelling (cd->html, TRUE);
 	}
 
 	if (html_engine_spell_word_is_valid (cd->html->engine))
-		if (next_word (cd)) {
+		if (next_word (cd, TRUE)) {
+			GtkWidget *info;
+
 			html_engine_hide_cursor (cd->html->engine);
 			html_cursor_jump_to_position (cd->html->engine->cursor, cd->html->engine, position);
 			html_engine_show_cursor (cd->html->engine);
 
-			/* FIX2 gnome_ok_dialog (_("No misspelled word found")); */
+			info = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
+						       GTK_BUTTONS_OK, _("No misspelled word found"));
+			gtk_dialog_run (GTK_DIALOG (info));
+			gtk_widget_destroy (info);
 
-			return;
+			goto end;
 		}
 
 	dialog  = gtk_dialog_new_with_buttons (_("Spell checker"), NULL, 0, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
@@ -269,28 +308,110 @@ spell_check_dialog (GtkHTMLControlData *cd, gboolean whole_document)
 	if (!control) {
 		g_warning ("Cannot create spell control");
 		gtk_widget_unref (dialog);
-		return;
+		goto end;
 	}
 
 	cd->spell_dialog = dialog;
         cd->spell_control_pb = bonobo_control_frame_get_control_property_bag
 		(bonobo_widget_get_control_frame (BONOBO_WIDGET (control)), NULL);
-	bonobo_property_bag_client_set_value_string (cd->spell_control_pb, "language",
-						     GTK_HTML_CLASS (GTK_OBJECT_GET_CLASS (cd->html))->properties->language,
-						     NULL);
+	bonobo_property_bag_client_set_value_string (cd->spell_control_pb, "language", html_engine_get_language (cd->html->engine), NULL);
+	bonobo_property_bag_client_set_value_gboolean (cd->spell_control_pb, "single", !whole_document, NULL);
 
 	bonobo_event_source_client_add_listener (cd->spell_control_pb, replace_cb, "Bonobo/Property:change:replace", NULL, cd);
 	bonobo_event_source_client_add_listener (cd->spell_control_pb, add_cb, "Bonobo/Property:change:add", NULL, cd);
-	bonobo_event_source_client_add_listener (cd->spell_control_pb, ignore_cb, "Bonobo/Property:change:ignore",
-						 NULL, cd);
+	bonobo_event_source_client_add_listener (cd->spell_control_pb, ignore_cb, "Bonobo/Property:change:ignore", NULL, cd);
 	bonobo_event_source_client_add_listener (cd->spell_control_pb, skip_cb, "Bonobo/Property:change:skip", NULL, cd);
+	bonobo_event_source_client_add_listener (cd->spell_control_pb, back_cb, "Bonobo/Property:change:back", NULL, cd);
 	set_word (cd);
 
 	gtk_widget_show (control);
-	gtk_container_add (GTK_CONTAINER (dialog), control);
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), control);
 	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
-	g_object_unref (dialog);
+	gtk_widget_destroy (dialog);
 	bonobo_object_release_unref (cd->spell_control_pb, NULL);
 	cd->spell_control_pb = CORBA_OBJECT_NIL;
+
+ end:
+	if (!inline_spelling)
+		gtk_html_set_inline_spelling (cd->html, inline_spelling);
+}
+
+static void
+language_cb (BonoboUIComponent *uic, const char *path, Bonobo_UIComponent_EventType type,
+	     const char *state, gpointer user_data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
+	GString *str, *lang;
+	gchar *val;
+	gint i;
+
+	if (cd->block_language_changes || !cd->languages)
+		return;
+
+	/* printf ("language callback %s\n", path); */
+
+	str = g_string_new (NULL);
+	lang = g_string_new (NULL);
+	for (i = 0; i < cd->languages->_length; i ++) {
+		g_string_printf (lang, "/commands/SpellLanguage%d", i + 1);
+		val = bonobo_ui_component_get_prop (cd->uic, lang->str, "state", NULL);
+		if (val && *val == '1') {
+			g_string_append (str, cd->languages->_buffer [i].abbreviation);
+			g_string_append_c (str, ' ');
+		}
+	}
+
+	html_engine_set_language (cd->html->engine, str->str);
+	g_free (cd->language);
+	cd->language = str->str;
+
+	g_string_free (str, FALSE);
+	g_string_free (lang, TRUE);
+}
+
+void
+spell_create_language_menu (GtkHTMLControlData *cd)
+{
+	CORBA_sequence_GNOME_Spell_Language *seq;
+	CORBA_Environment ev;
+
+	if (cd->dict == CORBA_OBJECT_NIL)
+		return;
+
+	if (cd->languages)
+		CORBA_free (cd->languages);
+
+	CORBA_exception_init (&ev);
+	cd->languages = seq = GNOME_Spell_Dictionary_getLanguages (cd->dict, &ev);
+
+	if (BONOBO_EX (&ev))
+		cd->languages = seq = NULL;
+
+	CORBA_exception_free (&ev);
+
+	if (seq && seq->_length > 0) {
+		GString *str;
+		gchar *line;
+		gint i;
+
+		str = g_string_new ("<submenu name=\"EditSpellLanguagesSubmenu\" _label=\"");
+		g_string_append (str, _("Current _Languages"));
+		g_string_append (str, "\">\n");
+		for (i = 0; i < seq->_length; i ++) {
+			line = g_strdup_printf ("<menuitem name=\"SpellLanguage%d\" _label=\"%s\""
+						" verb=\"SpellLanguage%d\" type=\"toggle\"/>\n",
+						i + 1, seq->_buffer [i].name, i + 1);
+			g_string_append (str, line);
+			g_free (line);
+		}
+		g_string_append (str, "</submenu>\n");
+
+		bonobo_ui_component_set_translate (cd->uic, "/menu/Edit/EditMisc/EditSpellLanguages/", str->str, NULL);
+
+		for (i = 0; i < seq->_length; i ++) {
+			g_string_printf (str, "SpellLanguage%d", i + 1);
+			bonobo_ui_component_add_listener (cd->uic, str->str, language_cb, cd);
+		}
+		g_string_free (str, TRUE);
+	}
 }
