@@ -22,28 +22,13 @@
 #ifndef _HTMLFONTSTYLE_H
 #define _HTMLFONTSTYLE_H
 
-enum _GtkHTMLFontStyle {
-	GTK_HTML_FONT_STYLE_DEFAULT = 0,
-	GTK_HTML_FONT_STYLE_SIZE_1 = 1,
-	GTK_HTML_FONT_STYLE_SIZE_2 = 2,
-	GTK_HTML_FONT_STYLE_SIZE_3 = 3,
-	GTK_HTML_FONT_STYLE_SIZE_4 = 4,
-	GTK_HTML_FONT_STYLE_SIZE_5 = 5,
-	GTK_HTML_FONT_STYLE_SIZE_6 = 6,
-	GTK_HTML_FONT_STYLE_SIZE_7 = 7,
-	GTK_HTML_FONT_STYLE_SIZE_MASK = 0x7,
-	GTK_HTML_FONT_STYLE_BOLD = 1 << 3,
-	GTK_HTML_FONT_STYLE_ITALIC = 1 << 4,
-	GTK_HTML_FONT_STYLE_UNDERLINE = 1 << 5,
-	GTK_HTML_FONT_STYLE_STRIKEOUT = 1 << 6,
-	GTK_HTML_FONT_STYLE_FIXED = 1 << 7
-};
-typedef enum _GtkHTMLFontStyle GtkHTMLFontStyle;
+#include "gtkhtml-enums.h"
 
 #define GTK_HTML_FONT_STYLE_SIZE_MAX 7
 #define GTK_HTML_FONT_STYLE_MAX									\
 	(GTK_HTML_FONT_STYLE_SIZE_MASK | GTK_HTML_FONT_STYLE_BOLD | GTK_HTML_FONT_STYLE_ITALIC	\
-	 | GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT | GTK_HTML_FONT_STYLE_FIXED)
+	 | GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT | GTK_HTML_FONT_STYLE_FIXED \
+	 | GTK_HTML_FONT_STYLE_SUBSCRIPT | GTK_HTML_FONT_STYLE_SUPERSCRIPT)
 #define GTK_HTML_FONT_STYLE_MAX_FONT      ((GTK_HTML_FONT_STYLE_ITALIC) << 1)
 #define GTK_HTML_FONT_STYLE_MAX_FONT_MASK (GTK_HTML_FONT_STYLE_MAX_FONT - 1)
 
