@@ -170,11 +170,6 @@ destroy (HTMLObject *o)
 	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
-static HTMLObject *
-op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GList *right, guint *len)
-{
-}
-
 static void
 copy (HTMLObject *self,
       HTMLObject *dest)
@@ -321,12 +316,14 @@ html_image_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_
 
 	pixel_size = html_painter_get_pixel_size (painter);
 
-	if (o->parent && HTML_IS_CLUEFLOW (o->parent)
-	    && HTML_IS_PLAIN_PAINTER (painter) && image->alt && *image->alt) {
-		GtkHTMLFontStyle style;
+	if (o->parent && HTML_IS_PLAIN_PAINTER (painter) && image->alt && *image->alt) {
+		GtkHTMLFontStyle style = GTK_HTML_FONT_STYLE_DEFAULT;
+		HTMLClueFlow *cf = html_object_get_flow (o);
 		gint lo = 0;
 
-		style = html_clueflow_get_default_font_style (HTML_CLUEFLOW (o->parent));
+		if (cf)
+			style = html_clueflow_get_default_font_style (cf);
+
 		/* FIXME: cache items and glyphs? */
 		html_painter_calc_text_size (painter, image->alt, g_utf8_strlen (image->alt, -1), NULL, NULL, NULL, 0, &lo,
 					     style, NULL, &o->width, &o->ascent, &o->descent);
@@ -359,6 +356,7 @@ draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint heig
 		return;
 
 	if (img->alt && *img->alt) {
+		HTMLClueFlow *cf = html_object_get_flow (o);
 
 		/* FIXME: cache items and glyphs? */
 		if (o->selected) {
@@ -373,6 +371,10 @@ draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint heig
 			html_painter_set_pen (p, &html_colorset_get_color_allocated (e->settings->color_set, p,
 										     HTMLTextColor)->color);
 		}
+		if (cf)
+			html_painter_set_font_style (p, html_clueflow_get_default_font_style (cf));
+
+  		html_painter_set_font_face (p, NULL);
 		html_painter_draw_text (p, o->x + tx, o->y + ty, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, NULL, 0, 0);
 	}
 }
@@ -1166,8 +1168,7 @@ html_image_pointer_queue_animation (HTMLImagePointer *ip)
 static gint
 html_image_pointer_update (HTMLImagePointer *ip)
 {
-	GdkPixbufAnimationIter  *iter = ip->iter;
-	HTMLEngine              *engine = ip->factory->engine;
+	HTMLEngine *engine = ip->factory->engine;
 	GSList *cur;
 
 	g_return_val_if_fail (ip->factory != NULL, FALSE);
@@ -1400,7 +1401,7 @@ html_image_pointer_unref (HTMLImagePointer *ip)
 	}
 }
 
-GtkHTMLStream *
+static GtkHTMLStream *
 html_image_pointer_load (HTMLImagePointer *ip)
 {
 	if (ip->factory->engine->stopped)
