@@ -41,6 +41,9 @@ struct _HTMLUndo {
 	GSList   *redo_levels;
 	guint     level;
 	guint     in_redo;
+	gint      step_counter;
+
+	gint      freeze_count; /* Freeze counter for im context */
 };
 
 #ifdef UNDO_DEBUG
@@ -92,6 +95,7 @@ html_undo_destroy  (HTMLUndo *undo)
 	g_return_if_fail (undo != NULL);
 
 	destroy_action_list (undo->undo.stack);
+	destroy_action_list (undo->undo_used.stack);
 	destroy_action_list (undo->redo.stack);
 
 	g_free (undo);
@@ -108,7 +112,8 @@ action_do_and_destroy_redo (HTMLEngine *engine, HTMLUndo *undo, GList **stack, H
 	action = HTML_UNDO_ACTION (first->data);
 
 	html_cursor_jump_to_position (engine->cursor, engine, action->position);
-	(* action->function) (engine, action->data, dir);
+	(* action->function) (engine, action->data, dir, action->position_after);
+	html_cursor_jump_to_position (engine->cursor, engine, action->position_after);
 
 	*stack = g_list_remove (first, first->data);
 	if (undo->level == 0) {
@@ -130,11 +135,14 @@ action_do_and_destroy_undo (HTMLEngine *engine, HTMLUndo *undo, HTMLUndoDirectio
 	action = HTML_UNDO_ACTION (first->data);
 
 	html_cursor_jump_to_position (engine->cursor, engine, action->position);
-	(* action->function) (engine, action->data, dir);
+	(* action->function) (engine, action->data, dir, action->position_after);
+	html_cursor_jump_to_position (engine->cursor, engine, action->position_after);
 
 	undo->undo.stack = g_list_remove (first, first->data);
-	if (undo->level == 0)
+	if (undo->level == 0) {
 		undo->undo_used.stack = g_list_prepend (undo->undo_used.stack, action);
+		undo->step_counter --;
+	}
 }
 
 void
@@ -144,6 +152,9 @@ html_undo_do_undo (HTMLUndo *undo,
 	g_return_if_fail (undo != NULL);
 	g_return_if_fail (engine != NULL);
 
+	if (undo->freeze_count > 0)
+		return;
+
 	if (undo->undo.size > 0) {
 #ifdef UNDO_DEBUG
 		if (!undo->level) {
@@ -171,6 +182,9 @@ html_undo_do_redo (HTMLUndo *undo,
 	g_return_if_fail (undo != NULL);
 	g_return_if_fail (engine != NULL);
 
+	if (undo->freeze_count > 0)
+		return;
+
 	if (undo->redo.size > 0) {
 #ifdef UNDO_DEBUG
 		if (!undo->level) {
@@ -200,6 +214,9 @@ html_undo_discard_redo (HTMLUndo *undo)
 {
 	g_return_if_fail (undo != NULL);
 
+	if (undo->freeze_count > 0)
+		return;
+
 	if (undo->redo.stack == NULL)
 		return;
 
@@ -215,22 +232,29 @@ html_undo_add_undo_action  (HTMLUndo *undo, HTMLUndoAction *action)
 	g_return_if_fail (undo != NULL);
 	g_return_if_fail (action != NULL);
 
-	if (undo->level == 0 && undo->in_redo == 0 && undo->redo.size > 0)
-		add_used_and_redo_to_undo (undo);
+	if (undo->freeze_count > 0)
+		return;
 
-	if (!undo->level && undo->undo.size >= HTML_UNDO_LIMIT) {
-		HTMLUndoAction *last_action;
-		GList *last;
+	if (undo->level == 0) {
+		if (undo->in_redo == 0 && undo->redo.size > 0)
+			add_used_and_redo_to_undo (undo);
 
-		last = g_list_last (undo->undo.stack);
-		last_action = (HTMLUndoAction *) last->data;
+		if (undo->undo.size >= HTML_UNDO_LIMIT) {
+			HTMLUndoAction *last_action;
+			GList *last;
 
-		undo->undo.stack = g_list_remove_link (undo->undo.stack, last);
-		g_list_free (last);
+			last = g_list_last (undo->undo.stack);
+			last_action = (HTMLUndoAction *) last->data;
 
-		html_undo_action_destroy (last_action);
+			undo->undo.stack = g_list_remove_link (undo->undo.stack, last);
+			g_list_free (last);
 
-		undo->undo.size--;
+			html_undo_action_destroy (last_action);
+
+			undo->undo.size--;
+		}
+
+		undo->step_counter ++;
 	}
 
 	undo->undo.stack = g_list_prepend (undo->undo.stack, action);
@@ -251,19 +275,37 @@ html_undo_add_redo_action  (HTMLUndo *undo,
 	g_return_if_fail (undo != NULL);
 	g_return_if_fail (action != NULL);
 
+	if (undo->freeze_count > 0)
+		return;
+
 	undo->redo.stack = g_list_prepend (undo->redo.stack, action);
-	undo->redo.size++;
+	undo->redo.size ++;
 }
 
 void
 html_undo_add_action  (HTMLUndo *undo, HTMLUndoAction *action, HTMLUndoDirection dir)
 {
+	if (undo->freeze_count > 0)
+		return;
+
 	if (dir == HTML_UNDO_UNDO)
 		html_undo_add_undo_action (undo, action);
 	else
 		html_undo_add_redo_action (undo, action);
 }
 
+void
+html_undo_freeze  (HTMLUndo *undo)
+{
+	undo->freeze_count ++;
+}
+
+void
+html_undo_thaw  (HTMLUndo *undo)
+{
+	undo->freeze_count --;
+}
+
 /*
   undo levels
 
@@ -292,7 +334,7 @@ struct _HTMLUndoLevel {
 };
 typedef struct _HTMLUndoLevel HTMLUndoLevel;
 
-static void undo_step_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir);
+static void undo_step_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, guint position_after);
 static void redo_level_begin (HTMLUndo *undo, const gchar *redo_desription, const gchar *undo_desription);
 static void redo_level_end   (HTMLUndo *undo);
 
@@ -380,7 +422,8 @@ redo_level_end (HTMLUndo *undo)
 		action = (HTMLUndoAction *) save_redo.stack->data;
 		html_undo_add_redo_action (undo, action = html_undo_action_new (level->description [HTML_UNDO_REDO],
 										undo_step_action,
-										HTML_UNDO_DATA (level), action->position));
+										HTML_UNDO_DATA (level),
+										action->position, action->position_after));
 #ifdef UNDO_DEBUG
 		action->is_level = TRUE;
 #endif
@@ -423,7 +466,8 @@ html_undo_level_end (HTMLUndo *undo)
 		action = html_undo_action_new (level->description [HTML_UNDO_UNDO],
 					       undo_step_action,
 					       HTML_UNDO_DATA (level),
-					       HTML_UNDO_ACTION (save_undo.stack->data)->position);
+					       HTML_UNDO_ACTION (save_undo.stack->data)->position,
+					       HTML_UNDO_ACTION (save_undo.stack->data)->position_after);
 #ifdef UNDO_DEBUG
 		action->is_level = TRUE;
 #endif
@@ -437,7 +481,7 @@ html_undo_level_end (HTMLUndo *undo)
 }
 
 static void
-undo_step_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir)
+undo_step_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, guint position_after)
 {
 	HTMLUndo      *undo;
 	HTMLUndoLevel *level;
@@ -576,3 +620,37 @@ html_undo_debug (HTMLUndo *undo)
 }
 
 #endif
+
+void
+html_undo_reset (HTMLUndo *undo)
+{
+	g_return_if_fail (undo != NULL);
+	g_return_if_fail (undo->level == 0);
+
+	destroy_action_list (undo->undo.stack);
+	destroy_action_list (undo->undo_used.stack);
+	destroy_action_list (undo->redo.stack);
+
+	undo->undo.stack = NULL;
+	undo->undo.size  = 0;
+	undo->undo_used.stack = NULL;
+	undo->undo_used.size  = 0;
+	undo->redo.stack = NULL;
+	undo->redo.size  = 0;
+
+	undo->step_counter = 0;
+}
+
+gboolean
+html_undo_has_undo_steps (HTMLUndo *undo)
+{
+	g_assert (undo->step_counter >= 0);
+
+	return undo->step_counter > 0;
+}
+
+gint
+html_undo_get_step_count (HTMLUndo *undo)
+{
+	return undo->step_counter;
+}
